__title__ = "pbipy"
__description__ = "A Power BI Rest API wrapper for Python."
__url__ = ""
__version__ = "0.1.0"
__author__ = "Andrew Villazon"
__author_email__ = "andrew.villazon@gmail.com"
__license__ = "Apache 2.0"
__copyright__ = "Copyright 2022 Andrew Villazon"